@@ -50,7 +50,7 @@ THE SOFTWARE.
 #include "xroute.h"
 #include "message.h"
 #include "resend.h"
-#include "config.h"
+#include "configuration.h"
 #include "local.h"
 
 struct timeval now;
@@ -89,7 +89,7 @@ static int kernel_addr_changed = 0;
 
 struct timeval check_neighbours_timeout;
 
-static volatile sig_atomic_t exiting = 0, dumping = 0, changed = 0;
+static volatile sig_atomic_t exiting = 0, dumping = 0, reopening = 0;
 
 int local_server_socket = -1, local_socket = -1;
 int local_server_port = -1;
@@ -125,7 +125,7 @@ main(int argc, char **argv)
     protocol_port = 6697;
 
     while(1) {
-        opt = getopt(argc, argv, "m:p:h:H:i:k:A:PsS:d:g:lwt:T:c:C:DL:I:");
+        opt = getopt(argc, argv, "m:p:h:H:i:k:A:PsuS:d:g:lwt:T:c:C:DL:I:");
         if(opt < 0)
             break;
 
@@ -180,6 +180,9 @@ main(int argc, char **argv)
         case 's':
             split_horizon = 0;
             break;
+        case 'u':
+            keep_unfeasible = 1;
+            break;
         case 'S':
             state_file = optarg;
             break;
@@ -619,7 +622,7 @@ main(int argc, char **argv)
         }
 #endif
 
-        if(changed) {
+        if(reopening) {
             kernel_dump_time = now.tv_sec;
             check_neighbours_timeout = now;
             expiry_time = now.tv_sec;
@@ -628,7 +631,7 @@ main(int argc, char **argv)
                 perror("reopen_logfile");
                 break;
             }
-            changed = 0;
+            reopening = 0;
         }
 
         if(kernel_link_changed || kernel_addr_changed) {
@@ -776,11 +779,11 @@ main(int argc, char **argv)
             "                "
             "[-h hello] [-H wired_hello] [-i idle_hello]\n"
             "                "
-            "[-k metric] [-A metric] [-s] [-P] [-l] [-w] [-d level] [-g port]\n"
+            "[-k metric] [-A metric] [-s] [-P] [-l] [-w] [-u] [-g port]\n"
             "                "
             "[-t table] [-T table] [-c file] [-C statement]\n"
             "                "
-            "[-D] [-L logfile] [-I pidfile]\n"
+            "[-d level] [-D] [-L logfile] [-I pidfile]\n"
             "                "
             "[id] interface...\n",
             argv[0]);
@@ -806,7 +809,7 @@ schedule_neighbours_check(int msecs, int override)
 {
     struct timeval timeout;
 
-    timeval_plus_msec(&timeout, &now, roughly(msecs * 3 / 2));
+    timeval_add_msec(&timeout, &now, roughly(msecs * 3 / 2));
     if(override)
         check_neighbours_timeout = timeout;
     else
@@ -852,9 +855,9 @@ sigdump(int signo)
 }
 
 static void
-sigchanged(int signo)
+sigreopening(int signo)
 {
-    changed = 1;
+    reopening = 1;
 }
 
 static void
@@ -894,7 +897,7 @@ init_signals(void)
     sigaction(SIGUSR1, &sa, NULL);
 
     sigemptyset(&ss);
-    sa.sa_handler = sigchanged;
+    sa.sa_handler = sigreopening;
     sa.sa_mask = ss;
     sa.sa_flags = 0;
     sigaction(SIGUSR2, &sa, NULL);
