@@ -35,7 +35,7 @@ THE SOFTWARE.
 #include "route.h"
 #include "xroute.h"
 #include "util.h"
-#include "config.h"
+#include "configuration.h"
 #include "network.h"
 #include "local.h"
 
